import javax.imageio.*;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.util.*;

public class EscPosPrinter {
    static final String APP_ID = "14df7284a4af4c72bb2dc0ca8b820a33";
    static final String APP_KEY = "d0959fea394e437786d8db0ec94977b4";

    static final int MAX_COLUMNS = 6;               // 分列打印最大列数

    static final int ALIGN_LEFT = 0;                // 左对齐
    static final int ALIGN_CENTER = 1;              // 居中对齐
    static final int ALIGN_RIGHT = 2;               // 右对齐

    static final int HRI_POS_ABOVE = 1;             // 条码文本在条码上方
    static final int HRI_POS_BELOW = 2;             // 条码文本在条码下方

    static final int COLUMN_FLAG_BW_REVERSE = 1;    // 分列打印标志：黑白反转

    int DOTS_PER_LINE = 384;                        // 一行的打印宽度(点)。58mm打印机为384，80mm打印机为576

    StringBuilder orderContent = new StringBuilder();
    int[] widthOfColumns = new int[MAX_COLUMNS];
    int[] flagOfColumns = new int[MAX_COLUMNS];
    int charHSize = 1;

    public EscPosPrinter() {
        DOTS_PER_LINE = 384;
    }

    public EscPosPrinter(int dotsPerLine) {
        DOTS_PER_LINE = (dotsPerLine == 576) ? 576 : 384;
    }

    public static int hexCharToInt(int c) {
        if (c >= '0' && c <= '9')
            c -= 48;
        else if (c >= 'A' && c <= 'F')
            c -= 55;
        else if (c >= 'a' && c <= 'f')
            c -= 87;
        else
            c = 0;
        return c;
    }

    public static byte[] hexStrToBytes(String str) {
        if (str == null || str.equals(""))
            return null;
        int length = str.length() / 2;
        if (length == 0)
            return null;
        byte[] bytes = new byte[length];
        int c, i, j;
        for (i = 0, j = 0; i < length; i++) {
            bytes[i]  = (byte)(hexCharToInt(str.charAt(j++)) << 4);
            bytes[i] |= (byte)(hexCharToInt(str.charAt(j++)));
        }
        return bytes;
    }

    public static String buildHttpQuery(Map<String, String> params) {
        Iterator<String> it = params.keySet().iterator();
        StringBuilder s = new StringBuilder();
        while (it.hasNext()) {
            String key = it.next();
            String value = params.get(key);
            try {
                value = URLEncoder.encode(value, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                ;
            }
            s.append(key).append("=").append(value).append("&");
        }
        return s.toString().substring(0, s.length() - 1);
    }

    public static String postPrintData(String httpUrl, Map<String, String> params) {
        HttpURLConnection connection = null;
        InputStream is = null;
        OutputStream os = null;
        BufferedReader br = null;
        String result = null;

        try {
            URL url = new URL(httpUrl);
            // 通过远程url连接对象打开连接
            connection = (HttpURLConnection)url.openConnection();
            // 设置连接请求方式
            connection.setRequestMethod("POST");
            // 设置连接主机服务器超时时间：15000毫秒
            connection.setConnectTimeout(15000);
            // 设置读取主机服务器返回数据超时时间：60000毫秒
            connection.setReadTimeout(60000);

            // 默认值为：false，当向远程服务器传送数据/写数据时，需要设置为true
            connection.setDoOutput(true);
            // 默认值为：true，当前向远程服务读取数据时，设置为true，该参数可有可无
            connection.setDoInput(true);
            // 设置传入参数的格式:请求参数应该是 name1=value1&name2=value2 的形式
            connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            // 通过连接对象获取一个输出流
            os = connection.getOutputStream();
            // 通过输出流对象将参数写出去/传输出去,它是通过字节数组写出的(form表单形式的参数实质也是key,value值的拼接，类似于get请求参数的拼接)
            os.write(buildHttpQuery(params).getBytes());
            // 通过连接对象获取一个输入流，向远程读取
            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                // 对输入流对象进行包装:charset根据工作项目组的要求来设置
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                StringBuffer sbf = new StringBuffer();
                String temp = null;
                // 循环遍历一行一行读取数据
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\n");
                }
                result = sbf.toString();
            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // 关闭资源
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != os) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 断开与远程地址url的连接
            connection.disconnect();
        }
        return result;
    }

    // 生成签名。
    public static String generateSign(Map<String, String> formData) {
        Set<String> keys = formData.keySet();
        Set<String> sortedKeys = new TreeSet<String>(Comparator.naturalOrder());
        sortedKeys.addAll(keys);

        StringBuilder s = new StringBuilder();
        for (String k: sortedKeys) {
            s.append(k + "=" + formData.get(k) + "&");
        }
        s.delete(s.length() - 1, s.length());
        s.append(APP_KEY);

        try {
            MessageDigest m = MessageDigest.getInstance("MD5");
            m.update(s.substring(0).getBytes("UTF-8"));
            byte[] r = m.digest();
            StringBuilder sign = new StringBuilder();
            for (int i = 0; i < r.length; i++) {
                sign.append(String.format("%02X", r[i]));
            }
            return sign.toString();
        } catch (Exception e) {
            return "";
        }
    }

    // 使用直推模式打印小票内容。
    public void pushToPrinter(String msn) {
        Date date = new Date();
        long timestamp = date.getTime() / 1000;
        int voiceCnt = 1;
        String voice = "您有新的订单";
        int orderCnt = 1;
        int orderType = 1;

        Map<String, String> formData = new HashMap<String, String>();
        formData.put("app_id",    APP_ID);
        formData.put("msn",       msn);
        formData.put("timestamp", String.format("%d", timestamp));
        formData.put("pushId",    String.format("%s_%010d", msn, timestamp));
        formData.put("voiceCnt",  String.format("%d", voiceCnt));
        formData.put("voice",     voice);
        formData.put("orderCnt",  String.format("%d", orderCnt));
        formData.put("orderType", String.format("%d", orderType));
        formData.put("orderData", orderContent.toString());
        formData.put("sign",      generateSign(formData));
        orderContent.delete(0, orderContent.length());

        String res = postPrintData("https://openapi.sunmi.com/v1/printer/pushContent", formData);
        if (res != null)
            System.out.println(res);
    }

    //////////////////////////////////////////////////
    // 基本ESC/POS指令
    //////////////////////////////////////////////////

    // 在小票内容中追加文本字符串。
    public void appendText(String str) {
        try {
            byte[] bytes = str.getBytes("UTF-8");
            for (int i = 0; i < bytes.length; i++)
                orderContent.append(String.format("%02x", bytes[i]));
        } catch (UnsupportedEncodingException e) {
            ;
        }
    }

    // [LF] 打印缓冲区里的内容并走纸 n 行。
    public void lineFeed(int n) {
        if (n <= 0)
            n = 1;
        for (int i = 0; i < n; i++)
            orderContent.append("0a");
    }

    // [ESC @] 恢复默认设置 (行间距、字符效果等)。
    public void restoreDefaultSettings() {
        charHSize = 1;
        orderContent.append("1b40");
    }

    // [ESC 2] 恢复默认的行间距。
    public void restoreDefaultLineSpacing() {
        orderContent.append("1b32");
    }

    // [ESC 3] 设置行间距。
    public void setLineSpacing(int n) {
        if (n >= 0 && n <= 255)
            orderContent.append("1b33" + String.format("%02x", n));
    }

    // [ESC !] 选择加粗、倍高、倍宽模式。
    public void setPrintModes(boolean bold, boolean double_h, boolean double_w) {
        int n = 0;
        if (bold)
            n |= 8;
        if (double_h)
            n |= 16;
        if (double_w)
            n |= 32;
        charHSize = (double_w) ? 2 : 1;
        orderContent.append("1b21" + String.format("%02x", n));
    }

    // [GS !] 选择字符高度和宽度倍数。
    public void setCharacterSize(int h, int w) {
        int n = 0;
        if (h >= 1 && h <= 8)
            n |= (h - 1);
        if (w >= 1 && w <= 8) {
            n |= (w - 1) << 4;
            charHSize = w;
        }
        orderContent.append("1d21" + String.format("%02x", n));
    }

    // [HT] 跳到下 n 个制表符位置。
    public void horizontalTab(int n) {
        for (int i = 0; i < n; i++)
            orderContent.append("09");
    }

    // [ESC $] 跳到绝对位置。
    public void setAbsolutePrintPosition(int n) {
        if (n >= 0 && n <= 65535)
            orderContent.append("1b24" + String.format("%02x%02x", (n & 0xff), ((n >> 8) & 0xff)));
    }

    // [ESC \] 跳到相对位置。
    public void setRelativePrintPosition(int n) {
        if (n >= -32768 && n <= 32767)
            orderContent.append("1b5c" + String.format("%02x%02x", (n & 0xff), ((n >> 8) & 0xff)));
    }

    // [ESC a] 设置对齐方式。
    public void setAlignment(int n) {
        if (n >= 0 && n <= 2)
            orderContent.append("1b61" + String.format("%02x", n));
    }

    // [GS B] 设置黑白反转模式。
    public void setBlackWhiteReverseMode(boolean enabled) {
        orderContent.append("1d42" + ((enabled) ? "01" : "00"));
    }

    // [ESC {] 设置上下颠倒模式。
    public void setUpsideDownMode(boolean enabled) {
        orderContent.append("1b7b" + ((enabled) ? "01" : "00"));
    }

    // [GS V m] 切纸。
    public void cutPaper(boolean fullCut) {
        orderContent.append("1d56" + ((fullCut) ? "30" : "31"));
    }

    // [GS V m n] 延迟切纸。
    // 发送该指令后，打印机不会马上切纸，而是等到后续走纸或打印了 (d + n) 点的距离后再切纸。
    // 其中 d 为切刀与打印头之间的距离，n 为 0~255。
    public void postponedCutPaper(boolean fullCut, int n) {
        if (n >= 0 && n <= 255)
            orderContent.append("1d56" + ((fullCut) ? "61" : "62") + String.format("%02x", n));
    }

    //////////////////////////////////////////////////
    // 分列打印
    //////////////////////////////////////////////////

    // 返回一个字符的宽度。0表示未知。
    public static int widthOfChar(int c) {
        if ((c >= 0x00020 && c <= 0x0036f) ||
            (c >= 0x0ff61 && c <= 0x0ff9f))
            return 12;
        if ((c == 0x02010                ) ||
            (c >= 0x02013 && c <= 0x02016) ||
            (c >= 0x02018 && c <= 0x02019) ||
            (c >= 0x0201c && c <= 0x0201d) ||
            (c >= 0x02025 && c <= 0x02026) ||
            (c >= 0x02030 && c <= 0x02033) ||
            (c == 0x02035                ) ||
            (c == 0x0203b                ))
            return 24;
        if ((c >= 0x01100 && c <= 0x011ff) ||
            (c >= 0x02460 && c <= 0x024ff) ||
            (c >= 0x025a0 && c <= 0x027bf) ||
            (c >= 0x02e80 && c <= 0x02fdf) ||
            (c >= 0x03000 && c <= 0x0318f) ||
            (c >= 0x031a0 && c <= 0x031ef) ||
            (c >= 0x03200 && c <= 0x09fff) ||
            (c >= 0x0ac00 && c <= 0x0d7ff) ||
            (c >= 0x0f900 && c <= 0x0faff) ||
            (c >= 0x0fe30 && c <= 0x0fe4f) ||
            (c >= 0x1f000 && c <= 0x1f9ff))
            return 24;
        if ((c >= 0x0ff01 && c <= 0x0ff5e) ||
            (c >= 0x0ffe0 && c <= 0x0ffe5))
            return 24;
        return 0;
    }

    public int widthOfString(String str) {
        int i, width = 0;

        for (i = 0; i < str.length(); i++) {
            width += widthOfChar((int)str.charAt(i));
        }
        return width * charHSize;
    }

    public static int columnWidthWithAlignment(int width, int alignment) {
        return (width & 0xffff) | ((alignment & 3) << 16);
    }

    public static int columnWidth(int v) {
        return v & 0xffff;
    }

    public static int columnAlignment(int v) {
        return (v >> 16) & 3;
    }

    // 设置每列的宽度和对齐方式。
    public void setColumnWidths(int... widths) {
        int i, remain, width, alignment;

        for (i = 0; i < MAX_COLUMNS; i++)
            widthOfColumns[i] = 0;

        remain = DOTS_PER_LINE; // 未分配的宽度
        for (i = 0; i < widths.length; i++) {
            if (i == MAX_COLUMNS) // 超过最大支持的列数
                return;
            width = columnWidth(widths[i]);
            alignment = columnAlignment(widths[i]);
            if (width == 0 || width > remain) { // 使用所有未分配的宽度作为最后一列
                widthOfColumns[i] = columnWidthWithAlignment(remain, alignment);
                return;
            }
            widthOfColumns[i] = widths[i];
            remain -= width;
        }
    }

    // 设置每列的标志位。
    public void setColumnFlags(int... flags) {
        int i;

        for (i = 0; i < flags.length; i++) {
            if (i == MAX_COLUMNS) // 超过最大支持的列数
                return;
            flagOfColumns[i] = flags[i];
        }
        for (; i < MAX_COLUMNS; i++)
            flagOfColumns[i] = 0;
    }

    // 按当前的列设置进行分列打印。
    public void printInColumns(String... texts) {
        StringBuilder[] strcurr = new StringBuilder[MAX_COLUMNS];
        String[] strrem = new String[MAX_COLUMNS];
        int[] strwidth = new int[MAX_COLUMNS];
        int i, j, c, w, columns, width, alignment, pos;
        boolean done;

        columns = 0;
        for (i = 0; i < texts.length; i++) {
            if (i == MAX_COLUMNS || widthOfColumns[i] == 0)
                break;
            strcurr[i] = new StringBuilder();
            strrem[i] = texts[i];
            columns++;
        }

        do {
            done = true;
            pos = 0;
            for (i = 0; i < columns; i++) {
                width = columnWidth(widthOfColumns[i]);
                if (strrem[i].length() == 0) {
                    pos += width;
                    continue;
                }
                done = false;
                strcurr[i].delete(0, strcurr[i].length());
                strwidth[i] = 0;
                for (j = 0; j < strrem[i].length(); j++) {
                    c = (int)strrem[i].charAt(j);
                    if (c == '\n') {
                        j++; // 丢弃换行符
                        break;
                    } else {
                        w = widthOfChar(c);
                        if (w == 0) {
                            c = '?';
                            w = 12;
                        }
                        w *= charHSize;
                        if (strwidth[i] + w > width) {
                            break;
                        } else {
                            strcurr[i].append((char)c);
                            strwidth[i] += w;
                        }
                    }
                }
                if (j < strrem[i].length())
                    strrem[i] = strrem[i].substring(j);
                else
                    strrem[i] = "";

                alignment = columnAlignment(widthOfColumns[i]);
                switch (alignment) {
                case 1:
                    setAbsolutePrintPosition(pos + (width - strwidth[i]) / 2);
                    break;
                case 2:
                    setAbsolutePrintPosition(pos + (width - strwidth[i]));
                    break;
                default:
                    setAbsolutePrintPosition(pos);
                    break;
                }
                if ((flagOfColumns[i] & COLUMN_FLAG_BW_REVERSE) != 0)
                    setBlackWhiteReverseMode(true);
                appendText(strcurr[i].toString());
                if ((flagOfColumns[i] & COLUMN_FLAG_BW_REVERSE) != 0)
                    setBlackWhiteReverseMode(false);
                pos += width;
            }
            if (!done)
                lineFeed(1);
        } while (!done);
    }

    //////////////////////////////////////////////////
    // 条码/二维码打印
    //////////////////////////////////////////////////

    // 在小票内容中追加条码。
    public void appendBarcode(int hri_pos, int height, int module_size, int barcode_type, String text) {
        int text_length = text.length();

        if (text_length == 0)
            return;
        if (text_length > 255)
            text_length = 255;
        if (height < 1)
            height = 1;
        else if (height > 255)
            height = 255;
        if (module_size < 1)
            module_size = 1;
        else if (module_size > 6)
            module_size = 6;

        orderContent.append("1d48" + String.format("%02x", (hri_pos & 3)));
        orderContent.append("1d6600");
        orderContent.append("1d68" + String.format("%02x", height));
        orderContent.append("1d77" + String.format("%02x", module_size));
        orderContent.append("1d6b" + String.format("%02x%02x", barcode_type, text_length));

        byte[] content = text.getBytes();
        for (int i = 0; i < text_length; i++)
            orderContent.append(String.format("%02x", content[i]));
    }

    // 在小票内容中追加二维码。
    public void appendQRcode(int module_size, int ec_level, String text) {
        byte[] content;
        int text_length;

        try {
            content = text.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            return;
        }

        text_length = content.length;

        if (text_length == 0)
            return;
        if (text_length > 65535)
            text_length = 65535;
        if (module_size < 1)
            module_size = 1;
        else if (module_size > 16)
            module_size = 16;
        if (ec_level < 0)
            ec_level = 0;
        else if (ec_level > 3)
            ec_level = 3;

        orderContent.append("1d286b040031410000");
        orderContent.append("1d286b03003143" + String.format("%02x", module_size));
        orderContent.append("1d286b03003145" + String.format("%02x", ec_level + 48));
        orderContent.append("1d286b" + String.format("%02x%02x315030", ((text_length + 3) & 0xff), (((text_length + 3) >> 8) & 0xff)));

        for (int i = 0; i < text_length; i++)
            orderContent.append(String.format("%02x", content[i]));

        orderContent.append("1d286b0300315130");
    }

    //////////////////////////////////////////////////
    // 图像打印
    //////////////////////////////////////////////////

    // 灰阶转单色 - 弥漫仿色算法。
    public static int[] diffuseDither(int[] src_data, int width, int height) {
        if (width <= 0 || height <= 0)
            return null;
        if (src_data.length < width * height)
            return null;

        int bmwidth = (width + 7) / 8;
        int[] dst_data = new int[bmwidth * height];

        int[][] line_buffer = new int[2][width];
        int i, p, q, x, y, mask;
        int line1, line2, b1, b2, tmp;
        int err, e1, e3, e5, e7;
        boolean not_last_line;

        line1 = 0;
        line2 = 1;

        for (i = 0; i < width; i++) {
            line_buffer[0][i] = 0;
            line_buffer[1][i] = src_data[i];
        }

        for (y = 0; y < height; y++) {
            tmp = line1;
            line1 = line2;
            line2 = tmp;
            not_last_line = (y < height - 1);

            if (not_last_line) {
                p = (y + 1) * width;
                for (i = 0; i < width; i++) {
                    line_buffer[line2][i] = src_data[p];
                    p++;
                }
            }

            q = y * bmwidth;
            for (i = 0; i < bmwidth; i++) {
                dst_data[q] = 0;
                q++;
            }

            b1 = 0;
            b2 = 0;
            q = y * bmwidth;
            mask = 0x80;

            for (x = 1; x <= width; x++) {
                if (line_buffer[line1][b1] < 128) { // Black pixel
                    err = line_buffer[line1][b1];
                    dst_data[q] |= mask;
                } else {
                    err = line_buffer[line1][b1] - 255;
                }
                b1++;
                if (mask == 1) {
                    q++;
                    mask = 0x80;
                } else {
                    mask >>= 1;
                }
                e7 = ((err * 7) + 8) >> 4;
                e5 = ((err * 5) + 8) >> 4;
                e3 = ((err * 3) + 8) >> 4;
                e1 = err - (e7 + e5 + e3);
                if (x < width)
                    line_buffer[line1][b1] += e7;
                if (not_last_line) {
                    line_buffer[line2][b2] += e5;
                    if (x > 1)
                        line_buffer[line2][b2 - 1] += e3;
                    if (x < width)
                        line_buffer[line2][b2 + 1] += e1;
                }
                b2++;
            }
        }

        return dst_data;
    }

    // 灰阶转单色 - 门限值仿色算法。
    public static int[] thresholdDither(int[] src_data, int width, int height) {
        if (width <= 0 || height <= 0)
            return null;
        if (src_data.length < width * height)
            return null;

        int bmwidth = (width + 7) / 8;
        int[] dst_data = new int[bmwidth * height];

        int p, q, k, x, y, mask;

        p = 0;
        q = 0;

        for (y = 0; y < height; y++) {
            k = q;
            mask = 0x80;
            for (x = 0; x < width; x++) {
                if (src_data[p] < 128) { // Black pixel
                    dst_data[k] |= mask;
                }
                p++;
                if (mask == 1) {
                    k++;
                    mask = 0x80;
                } else {
                    mask >>= 1;
                }
            }
            q += bmwidth;
        }

        return dst_data;
    }

    // RGB转灰阶。
    public static int[] convertToGray(BufferedImage img, int width, int height) {
        int [] gray_data = new int[width * height];
        int i = 0, x, y, rgb, r, g, b;

        for (y = 0; y < height; y++) {
            for (x = 0; x < width; x++) {
                rgb = img.getRGB(x, y);
                r = (rgb >> 16) & 0xff;
                g = (rgb >> 8 ) & 0xff;
                b = (rgb      ) & 0xff;
                gray_data[i] = ((r * 11 + g * 16 + b * 5) / 32) & 0xff;
                i++;
            }
        }
        return gray_data;
    }

    // 在小票内容中追加图片。
    public void appendImage(String imageFile) {
        BufferedImage imgOrg, imgRes;
        int i, w, h;

        try {
            imgOrg = ImageIO.read(new File(imageFile));
        } catch (IOException e) {
            return;
        }

        w = imgOrg.getWidth();
        h = imgOrg.getHeight();
        if (w > DOTS_PER_LINE) {
            h = DOTS_PER_LINE * h / w;
            w = DOTS_PER_LINE;
            imgRes = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
            Image img = imgOrg.getScaledInstance(w, h, Image.SCALE_SMOOTH);
            Graphics2D graphics = imgRes.createGraphics();
            graphics.drawImage(img, 0, 0, null);
            graphics.dispose();
        } else {
            imgRes = imgOrg;
        }

        int[] gray_data, mono_data;

        gray_data = convertToGray(imgRes, w, h);
        mono_data = diffuseDither(gray_data, w, h);
//      mono_data = thresholdDither(gray_data, w, h);

        w = (w + 7) / 8;
        orderContent.append("1d763000");
        orderContent.append(String.format("%02x%02x", (w & 0xff), ((w >> 8) & 0xff)));
        orderContent.append(String.format("%02x%02x", (h & 0xff), ((h >> 8) & 0xff)));
        for (i = 0; i < mono_data.length; i++)
            orderContent.append(String.format("%02x", (mono_data[i] & 0xff)));
    }

    //////////////////////////////////////////////////
    // 页模式指令
    //////////////////////////////////////////////////

    // 进入页模式。
    public void enterPageMode() {
        orderContent.append("1b4c");
    }

    // 设置页模式打印区域。
    // x, y: 打印区域原点（区域的左上角）坐标
    // w, h: 打印区域的宽度和高度
    public void setPrintAreaInPageMode(int x, int y, int w, int h) {
        orderContent.append("1b57");
        orderContent.append(String.format("%02x%02x", (x & 0xff), ((x >> 8) & 0xff)));
        orderContent.append(String.format("%02x%02x", (y & 0xff), ((y >> 8) & 0xff)));
        orderContent.append(String.format("%02x%02x", (w & 0xff), ((w >> 8) & 0xff)));
        orderContent.append(String.format("%02x%02x", (h & 0xff), ((h >> 8) & 0xff)));
    }

    // 设置页模式打印方向。
    // dir: 0:画布不旋转; 1:画布顺时针旋转90度; 2:画布顺时针旋转180度; 3:画布顺时针旋转270度
    public void setPrintDirectionInPageMode(int dir) {
        if (dir >= 0 && dir <= 3)
            orderContent.append("1b54" + String.format("%02x", dir));
    }

    // 设置页模式下的绝对垂直打印位置。
    public void setAbsolutePrintPositionInPageMode(int n) {
        if (n >= 0 && n <= 65535)
            orderContent.append("1d24" + String.format("%02x%02x", (n & 0xff), ((n >> 8) & 0xff)));
    }

    // 设置页模式下的相对垂直打印位置。
    public void setRelativePrintPositionInPageMode(int n) {
        if (n >= -32768 && n <= 32767)
            orderContent.append("1d5c" + String.format("%02x%02x", (n & 0xff), ((n >> 8) & 0xff)));
    }

    // 打印并退出页模式。
    public void printAndExitPageMode() {
        orderContent.append("0c");
    }

    // 打印（维持在页模式）。
    public void printInPageMode() {
        orderContent.append("1b0c");
    }

    // 清除打印区域（维持在页模式）。
    public void clearInPageMode() {
        orderContent.append("18");
    }

    // 不打印，退出页模式。
    public void exitPageMode() {
        orderContent.append("1b53");
    }

    public static void main(String[] args) {
        EscPosPrinter printer = new EscPosPrinter();

        printer.lineFeed(1);

        printer.setLineSpacing(80);
        printer.setPrintModes(true, true, false);
        printer.setAlignment(ALIGN_CENTER);
        printer.appendText("*** 打印测试 ***\n");

        printer.restoreDefaultLineSpacing();
        printer.setPrintModes(false, false, false);
        printer.setAlignment(ALIGN_LEFT);

        /* 设置三列。第一列96点，左对齐；第二列144点，居中对齐；第三列使用剩余的宽度，右对齐 */
        printer.setColumnWidths(
            columnWidthWithAlignment(96,  ALIGN_LEFT  ),
            columnWidthWithAlignment(144, ALIGN_CENTER),
            columnWidthWithAlignment(0,   ALIGN_RIGHT ));
        printer.setColumnFlags(
            0,
            0,
            COLUMN_FLAG_BW_REVERSE);
        printer.printInColumns("商品名称", "数量\n(单位：随意)", "小计\n(单位：元)");
        printer.lineFeed(1);
        printer.printInColumns("这是\"一个很长的品名\"", "x1000", "￥2020.99");
        printer.lineFeed(1);
        printer.printInColumns("橙子", "【备注：赠品购物满1000,000元送一只】", "￥0.00");
        printer.lineFeed(1);

        printer.setAlignment(ALIGN_CENTER);

        /* 打印 CODE128 条码 */
        printer.appendBarcode(HRI_POS_BELOW, 160, 3, 73, "Abc-000789");
        printer.lineFeed(1);

        /* 打印二维码 */
        printer.appendQRcode(5, 1, "https://docs.sunmi.com/docking-debugging/商米云打印机合作伙伴对接说明/1-了解一下对接流程/");
        printer.lineFeed(1);

        printer.setAlignment(ALIGN_LEFT);

        /* 打印图像文件 */
        if (args.length > 0)
            printer.appendImage(args[0]);
        printer.lineFeed(1);

        /* 页模式打印 */
        printer.setAlignment(ALIGN_CENTER);
        printer.appendText("---- 页模式多区域打印 ----\n");
        printer.setAlignment(ALIGN_LEFT);
        printer.enterPageMode();
        // 区域1
        printer.setPrintAreaInPageMode(0, 0, 144, 500);
        printer.setPrintDirectionInPageMode(0);
        printer.appendText("永和九年，岁在癸丑，暮春之初，会于会稽山阴之兰亭，修禊事也。群贤毕至，少长咸集。" +
            "此地有崇山峻岭，茂林修竹；又有清流激湍，映带左右，引以为流觞曲水，列坐其次。\n");
        // 区域2
        printer.setPrintAreaInPageMode(156, 0, 144, 500);
        printer.setPrintDirectionInPageMode(2);
        printer.appendText("鎌倉アナウンサーはまず流暢な中国語でアナウンサーとしての豊富な経験を紹介されました。\n");
        // 区域3
        printer.setPrintAreaInPageMode(312, 0, 72, 500);
        printer.setPrintDirectionInPageMode(3);
        printer.appendText("Scarlett is a woman who can deal with a nation at war, Atlanta burning.\n");
        // 打印并退出页模式
        printer.printAndExitPageMode();

        printer.lineFeed(4);
        printer.cutPaper(false);

        printer.pushToPrinter("N403D06A00023");
    }
}
