<?php

$APP_ID = "14df7284a4af4c72bb2dc0ca8b820a33";
$APP_KEY = "d0959fea394e437786d8db0ec94977b4";

define("DOTS_PER_LINE", 384);           // 一行的打印宽度(点)。58mm打印机为384，80mm打印机为576
define("MAX_COLUMNS",   6);             // 分列打印最大列数

define("ALIGN_LEFT",    0);             // 左对齐
define("ALIGN_CENTER",  1);             // 居中对齐
define("ALIGN_RIGHT",   2);             // 右对齐

define("HRI_POS_ABOVE", 1);             // 条码文本在条码上方
define("HRI_POS_BELOW", 2);             // 条码文本在条码下方

define("COLUMN_FLAG_BW_REVERSE", 1);    // 分列打印标志：黑白反转

$widthOfColumns = array_fill(0, MAX_COLUMNS, 0);
$flagOfColumns = array_fill(0, MAX_COLUMNS, 0);
$charHSize = 1;

/*
 * 生成签名。
 */
function generateSign($formData)
{
    global $APP_KEY;

    ksort($formData);
    $s = "";
    while (list($key, $val) = each($formData)) {
        $s .= $key . "=" . $val . "&";
    }
    $s = substr($s, 0, -1);
    $s .= $APP_KEY;
    return strtoupper(md5($s));
}

function request_by_curl($url, $params)
{
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
    $res = curl_exec($ch);
    if (curl_errno($ch)) {
        $res = curl_error($ch);
    }
    curl_close($ch);
    return $res;
}

/*
 * 使用直推模式打印小票内容。
 */
function pushToPrinter($msn, $orderData)
{
    global $APP_ID;

    $timestamp = time();
    $voiceCnt = 1;
    $voice = "您有新的订单";
    $orderCnt = 1;
    $orderType = 1;

    $formData = [
        "app_id"    => $APP_ID,
        "msn"       => $msn,
        "timestamp" => $timestamp,
        "pushId"    => sprintf("%s_%010d", $msn, $timestamp),
        "voiceCnt"  => $voiceCnt,
        "voice"     => $voice,
        "orderCnt"  => $orderCnt,
        "orderType" => $orderType,
        "orderData" => $orderData
    ];

    $formData["sign"] = generateSign($formData);
    $res = request_by_curl("https://openapi.sunmi.com/v1/printer/pushContent", http_build_query($formData));
    echo $res . "<br />";
}

//////////////////////////////////////////////////
// 基本ESC/POS指令
//////////////////////////////////////////////////

/*
 * 在小票内容中追加文本字符串。
 */
function appendText($str)
{
    $result = "";
    for ($i = 0; $i < strlen($str); $i++)
        $result .= sprintf("%02x", ord($str[$i]));
    return $result;
}

/*
 * [LF] 打印缓冲区里的内容并走纸 n 行。
 */
function lineFeed($n)
{
    $result = "";
    for ($i = 0; $i < $n; $i++)
        $result .= "0a";
    return $result;
}

/*
 * [ESC @] 恢复默认设置 (行间距、字符效果等)。
 */
function restoreDefaultSettings()
{
    global $charHSize;

    $charHSize = 1;
    return "1b40";
}

/*
 * [ESC 2] 恢复默认的行间距。
 */
function restoreDefaultLineSpacing()
{
    return "1b32";
}

/*
 * [ESC 3] 设置行间距。
 */
function setLineSpacing($n)
{
    if ($n >= 0 && $n <= 255)
        return sprintf("1b33%02x", $n);
    return "";
}

/*
 * [ESC !] 选择加粗、倍高、倍宽模式。
 */
function setPrintModes($bold, $double_h, $double_w)
{
    global $charHSize;

    $n = 0;
    if ($bold)
        $n |= 8;
    if ($double_h)
        $n |= 16;
    if ($double_w)
        $n |= 32;
    $charHSize = ($double_w) ? 2 : 1;
    return sprintf("1b21%02x", $n);
}

/*
 * [GS !] 选择字符高度和宽度倍数。
 */
function setCharacterSize($h, $w)
{
    global $charHSize;

    $n = 0;
    if ($h >= 1 && $h <= 8)
        $n |= ($h - 1);
    if ($w >= 1 && $w <= 8) {
        $n |= ($w - 1) << 4;
        $charHSize = $w;
    }
    return sprintf("1d21%02x", $n);
}

/*
 * [HT] 跳到下 n 个制表符位置。
 */
function horizontalTab($n)
{
    $result = "";
    for ($i = 0; $i < $n; $i++)
        $result .= "09";
    return $result;
}

/*
 * [ESC $] 跳到绝对位置。
 */
function setAbsolutePrintPosition($n)
{
    if ($n >= 0 && $n <= 65535)
        return sprintf("1b24%02x%02x", ($n & 0xFF), (($n >> 8) & 0xFF));
    return "";
}

/*
 * [ESC \] 跳到相对位置。
 */
function setRelativePrintPosition($n)
{
    if ($n >= -32768 && $n <= 32767)
        return sprintf("1b5c%02x%02x", ($n & 0xFF), (($n >> 8) & 0xFF));
    return "";
}

/*
 * [ESC a] 设置对齐方式。
 */
function setAlignment($n)
{
    if ($n >= 0 && $n <= 2)
        return sprintf("1b61%02x", $n);
    return "";
}

/*
 * [GS B] 设置黑白反转模式。
 */
function setBlackWhiteReverseMode($enabled)
{
    return sprintf("1d42%02x", ($enabled) ? 1 : 0);
}

/*
 * [ESC {] 设置上下颠倒模式。
 */
function setUpsideDownMode($enabled)
{
    return sprintf("1b7b%02x", ($enabled) ? 1 : 0);
}

/*
 * [GS V m] 切纸。
 */
function cutPaper($fullCut)
{
    return sprintf("1d56%02x", ($fullCut) ? 48 : 49);
}

/*
 * [GS V m n] 延迟切纸。
 * 发送该指令后，打印机不会马上切纸，而是等到后续走纸或打印了 (d + n) 点的距离后再切纸。
 * 其中 d 为切刀与打印头之间的距离，n 为 0~255。
 */
function postponedCutPaper($fullCut, $n)
{
    if ($n >= 0 && $n <= 255)
        return sprintf("1d56%02x%02x", ($fullCut) ? 97 : 98, $n);
    return "";
}

//////////////////////////////////////////////////
// 分列打印
//////////////////////////////////////////////////

function utf8_to_unicode($str, $size, &$unicode)
{
    $unicode = 0;

    if ($size < 1)
        return 0;
    $v0 = ord($str[0]);
    if (($v0 & 0x80) == 0x00) {
        $unicode = $v0;
        return 1;
    }

    if ($size < 2)
        return $size;
    $v1 = ord($str[1]);
    if (($v0 & 0xE0) == 0xC0 &&
        ($v1 & 0xC0) == 0x80) {
        $unicode = (($v0 & 0x1F) << 6) + ($v1 & 0x3F);
        return 2;
    }

    if ($size < 3)
        return $size;
    $v2 = ord($str[2]);
    if (($v0 & 0xF0) == 0xE0 &&
        ($v1 & 0xC0) == 0x80 &&
        ($v2 & 0xC0) == 0x80) {
        $unicode = (($v0 & 0x0F) << 12) + (($v1 & 0x3F) << 6) + ($v2 & 0x3F);
        return 3;
    }

    if ($size < 4)
        return $size;
    $v3 = ord($str[3]);
    if (($v0 & 0xF8) == 0xF0 &&
        ($v1 & 0xC0) == 0x80 &&
        ($v2 & 0xC0) == 0x80 &&
        ($v3 & 0xC0) == 0x80) {
        $unicode = (($v0 & 0x07) << 18) + (($v1 & 0x3F) << 12) + (($v2 & 0x3F) << 6) + ($v3 & 0x3F);
        return 4;
    }

    return 1;
}

/*
 * 返回一个字符的宽度。0表示未知。
 */
function widthOfChar($c)
{
    if (($c >= 0x00020 && $c <= 0x0036F) ||
        ($c >= 0x0FF61 && $c <= 0x0FF9F))
        return 12;
    if (($c == 0x02010                 ) ||
        ($c >= 0x02013 && $c <= 0x02016) ||
        ($c >= 0x02018 && $c <= 0x02019) ||
        ($c >= 0x0201C && $c <= 0x0201D) ||
        ($c >= 0x02025 && $c <= 0x02026) ||
        ($c >= 0x02030 && $c <= 0x02033) ||
        ($c == 0x02035                 ) ||
        ($c == 0x0203B                 ))
        return 24;
    if (($c >= 0x01100 && $c <= 0x011FF) ||
        ($c >= 0x02460 && $c <= 0x024FF) ||
        ($c >= 0x025A0 && $c <= 0x027BF) ||
        ($c >= 0x02E80 && $c <= 0x02FDF) ||
        ($c >= 0x03000 && $c <= 0x0318F) ||
        ($c >= 0x031A0 && $c <= 0x031EF) ||
        ($c >= 0x03200 && $c <= 0x09FFF) ||
        ($c >= 0x0AC00 && $c <= 0x0D7FF) ||
        ($c >= 0x0F900 && $c <= 0x0FAFF) ||
        ($c >= 0x0FE30 && $c <= 0x0FE4F) ||
        ($c >= 0x1F000 && $c <= 0x1F9FF))
        return 24;
    if (($c >= 0x0FF01 && $c <= 0x0FF5E) ||
        ($c >= 0x0FFE0 && $c <= 0x0FFE5))
        return 24;
    return 0;
}

function widthOfString($str)
{
    global $charHSize;

    $w = 0;
    $i = 0;
    while ($i < strlen($str)) {
        $s = substr($str, $i);
        $i += utf8_to_unicode($s, strlen($s), $c);
        $w += widthOfChar($c) * $charHSize;
    }
    return $w;
}

function columnWidthWithAlignment($width, $alignment)
{
    return ($width & 0xFFFF) | (($alignment & 3) << 16);
}

function columnWidth($v)
{
    return $v & 0xFFFF;
}

function columnAlignment($v)
{
    return ($v >> 16) & 3;
}

/*
 * 设置每列的宽度和对齐方式。
 */
function setColumnWidths()
{
    global $widthOfColumns;

    for ($i = 0; $i < MAX_COLUMNS; $i++)
        $widthOfColumns[$i] = 0;

    $remain = DOTS_PER_LINE; // 未分配的宽度
    for ($i = 0; $i < func_num_args(); $i++) {
        if ($i == MAX_COLUMNS) // 超过最大支持的列数
            return;
        $wa = func_get_arg($i);
        $width = columnWidth($wa);
        $alignment = columnAlignment($wa);
        if ($width == 0 || $width > $remain) { // 使用所有未分配的宽度作为最后一列
            $widthOfColumns[$i] = columnWidthWithAlignment($remain, $alignment);
            return;
        }
        $widthOfColumns[$i] = $wa;
        $remain -= $width;
    }
}

/*
 * 设置每列的标志位。
 */
function setColumnFlags()
{
    global $flagOfColumns;

    for ($i = 0; $i < MAX_COLUMNS; $i++)
        $flagOfColumns[$i] = 0;

    for ($i = 0; $i < func_num_args(); $i++) {
        if ($i == MAX_COLUMNS) // 超过最大支持的列数
            return;
        $flagOfColumns[$i] = func_get_arg($i);
    }
}

/*
 * 按当前的列设置进行分列打印。
 */
function printInColumns()
{
    global $widthOfColumns, $flagOfColumns, $charHSize;

    $strcurr  = array_fill(0, MAX_COLUMNS, "");
    $strrem   = array_fill(0, MAX_COLUMNS, "");
    $strwidth = array_fill(0, MAX_COLUMNS, 0);

    $columns = 0;
    for ($i = 0; $i < func_num_args(); $i++) {
        if ($i == MAX_COLUMNS || $widthOfColumns[$i] == 0)
            break;
        $strrem[$i] = func_get_arg($i);
        $columns++;
    }

    $result = "";
    do {
        $done = true;
        $pos = 0;
        for ($i = 0; $i < $columns; $i++) {
            $width = columnWidth($widthOfColumns[$i]);
            if (strlen($strrem[$i]) == 0) {
                $pos += $width;
                continue;
            }
            $done = false;
            $strcurr[$i] = "";
            $strwidth[$i] = 0;
            while (strlen($strrem[$i]) > 0) {
                $bytes = utf8_to_unicode($strrem[$i], strlen($strrem[$i]), $c);
                if ($c == 0x0a) {
                    $strrem[$i] = substr($strrem[$i], 1);
                    break;
                } else {
                    $w = widthOfChar($c);
                    $w *= $charHSize;
                    if ($strwidth[$i] + $w > $width) {
                        break;
                    } else {
                        $strcurr[$i] .= substr($strrem[$i], 0, $bytes);
                        $strwidth[$i] += $w;
                        $strrem[$i] = substr($strrem[$i], $bytes);
                    }
                }
            }

            $alignment = columnAlignment($widthOfColumns[$i]);
            switch ($alignment) {
            case 1:
                $result .= setAbsolutePrintPosition($pos + ($width - $strwidth[$i]) / 2);
                break;
            case 2:
                $result .= setAbsolutePrintPosition($pos + ($width - $strwidth[$i]));
                break;
            default:
                $result .= setAbsolutePrintPosition($pos);
                break;
            }
            if (($flagOfColumns[$i] & COLUMN_FLAG_BW_REVERSE) != 0)
                $result .= setBlackWhiteReverseMode(true);
            $result .= appendText($strcurr[$i]);
            if (($flagOfColumns[$i] & COLUMN_FLAG_BW_REVERSE) != 0)
                $result .= setBlackWhiteReverseMode(false);
            $pos += $width;
        }
        if (!$done)
            $result .= lineFeed(1);
    } while (!$done);

    return $result;
}

//////////////////////////////////////////////////
// 条码/二维码打印
//////////////////////////////////////////////////

/*
 * 在小票内容中追加条码。
 */
function appendBarcode($hri_pos, $height, $module_size, $barcode_type, $text)
{
    $text_length = strlen($text);
    if ($text_length == 0)
        return "";
    if ($text_length > 255)
        $text_length = 255;
    if ($height < 1)
        $height = 1;
    else if ($height > 255)
        $height = 255;
    if ($module_size < 1)
        $module_size = 1;
    else if ($module_size > 6)
        $module_size = 6;

    $result  = "";
    $result .= sprintf("1d48%02x", ($hri_pos & 3));
    $result .= sprintf("1d6600");
    $result .= sprintf("1d68%02x", $height);
    $result .= sprintf("1d77%02x", $module_size);
    $result .= sprintf("1d6b%02x%02x", $barcode_type, $text_length);

    for ($i = 0; $i < $text_length; $i++)
        $result .= sprintf("%02x", ord($text[$i]));

    return $result;
}

/*
 * 在小票内容中追加二维码。
 */
function appendQRcode($module_size, $ec_level, $text)
{
    $text_length = strlen($text);
    if ($text_length == 0)
        return "";
    if ($text_length > 65535)
        $text_length = 65535;
    if ($module_size < 1)
        $module_size = 1;
    else if ($module_size > 16)
        $module_size = 16;
    if ($ec_level < 0)
        $ec_level = 0;
    else if ($ec_level > 3)
        $ec_level = 3;

    $result  = "";
    $result .= "1d286b040031410000";
    $result .= sprintf("1d286b03003143%02x", $module_size);
    $result .= sprintf("1d286b03003145%02x", $ec_level + 48);
    $result .= sprintf("1d286b%02x%02x315030", (($text_length + 3) & 0xFF), ((($text_length + 3) >> 8) & 0xFF));

    for ($i = 0; $i < $text_length; $i++)
        $result .= sprintf("%02x", ord($text[$i]));

    $result .= "1d286b0300315130";

    return $result;
}

//////////////////////////////////////////////////
// 图像打印
//////////////////////////////////////////////////

/*
 * 灰阶转单色 - 弥漫仿色算法。
 */
function diffuseDither($src_data, $width, $height)
{
    $line1 = 0;
    $line2 = 1;
    $bmwidth = intval(($width + 7) / 8);

    $dst_data = array_fill(0, $bmwidth * $height, 0);
    $linebuf[0] = array_fill(0, $width * $height, 0);
    $linebuf[1] = array_fill(0, $width * $height, 0);

    for ($x = 0; $x < $width; $x++)
        $linebuf[1][$x] = $src_data[$x];

    for ($y = 0; $y < $height; $y++) {
        $tmp = $line1;
        $line1 = $line2;
        $line2 = $tmp;
        $not_last_line = ($y < $height - 1) ? true : false;
        if ($not_last_line) {
            $p = ($y + 1) * $width;
            for ($x = 0; $x < $width; $x++)
                $linebuf[$line2][$x] = $src_data[$p + $x];
        }

        $q = $bmwidth * $y;
        for ($i = 0; $i < $bmwidth; $i++)
            $dst_data[$q + $i] = 0;
        $b1 = 0;
        $b2 = 0;
        $mask = 0x80;
        for ($x = 1; $x <= $width; $x++) {
            if ($linebuf[$line1][$b1] < 128) { // black pixel
                $err = $linebuf[$line1][$b1++];
                $dst_data[$q] |= $mask;
            } else { // white pixel
                $err = $linebuf[$line1][$b1++] - 255;
            }
            if ($mask == 1) {
                $q++;
                $mask = 0x80;
            } else {
                $mask >>= 1;
            }
            $e7 = (($err * 7) + 8) >> 4;
            $e5 = (($err * 5) + 8) >> 4;
            $e3 = (($err * 3) + 8) >> 4;
            $e1 = $err - ($e7 + $e5 + $e3);
            if ($x < $width)
                $linebuf[$line1][$b1] += $e7; // spread error to right pixel
            if ($not_last_line) {
                $linebuf[$line2][$b2] += $e5; // pixel below
                if ($x > 1)
                    $linebuf[$line2][$b2 - 1] += $e3; // pixel below left
                if ($x < $width)
                    $linebuf[$line2][$b2 + 1] += $e1; // pixel below right
            }
            $b2++;
        }
    }

    $dst_data = array_pad($dst_data, -(count($dst_data) + 8), 0);
    $dst_data[0] = 0x1d;
    $dst_data[1] = 0x76;
    $dst_data[2] = 0x30;
    $dst_data[3] = 0x00;
    $dst_data[4] = ($bmwidth     ) & 0xFF;
    $dst_data[5] = ($bmwidth >> 8) & 0xFF;
    $dst_data[6] = ($height      ) & 0xFF;
    $dst_data[7] = ($height  >> 8) & 0xFF;

    $result = "";
    for ($i = 0; $i < count($dst_data); $i++)
        $result .= sprintf("%02x", $dst_data[$i]);

    return $result;
}

/*
 * 灰阶转单色 - 门限值仿色算法。
 */
function thresholdDither($src_data, $width, $height)
{
    $bmwidth = intval(($width + 7) / 8);

    $dst_data = array_fill(0, $bmwidth * $height, 0);

    $p = 0;
    $q = 0;
    for ($y = 0; $y < $height; $y++) {
        $mask = 0x80;
        $k = 0;
        for ($x = 0; $x < $width; $x++) {
            if ($src_data[$p + $x] < 128) // black pixel
                $dst_data[$q + $k] |= $mask;
            if ($mask == 1) {
                $k++;
                $mask = 0x80;
            } else {
                $mask >>= 1;
            }
        }
        $p += $width;
        $q += $bmwidth;
    }

    $dst_data = array_pad($dst_data, -(count($dst_data) + 8), 0);
    $dst_data[0] = 0x1d;
    $dst_data[1] = 0x76;
    $dst_data[2] = 0x30;
    $dst_data[3] = 0x00;
    $dst_data[4] = ($bmwidth     ) & 0xFF;
    $dst_data[5] = ($bmwidth >> 8) & 0xFF;
    $dst_data[6] = ($height      ) & 0xFF;
    $dst_data[7] = ($height  >> 8) & 0xFF;

    $result = "";
    for ($i = 0; $i < count($dst_data); $i++)
        $result .= sprintf("%02x", $dst_data[$i]);

    return $result;
}

/*
 * 在小票内容中追加图像。
 */
function appendImage($filename)
{
    list($org_width, $org_height, $type) = getimagesize($filename);

    switch ($type) {
    case 1: // GIF
        $org_image = imagecreatefromgif($filename);
        break;
    case 2: // JPG
        $org_image = imagecreatefromjpeg($filename);
        break;
    case 3: // PNG
        $org_image = imagecreatefrompng($filename);
        break;
    case 6: // BMP
        $org_image = imagecreatefrombmp($filename);
        break;
    default:
        return "";
    }

    $w = $org_width;
    $h = $org_height;

    if ($w > DOTS_PER_LINE) {
        $h = DOTS_PER_LINE * $h / $w;
        $w = DOTS_PER_LINE;
    }

    $image = imagecreatetruecolor($w, $h);
    imagecopyresampled($image, $org_image, 0, 0, 0, 0, $w, $h, $org_width, $org_height);

    $i = 0;

    for ($y = 0; $y < $h; $y++) {
        for ($x = 0; $x < $w; $x++) {
            $rgb = imagecolorat($image, $x, $y);
            $r = ($rgb >> 16) & 0xFF;
            $g = ($rgb >> 8 ) & 0xFF;
            $b = ($rgb      ) & 0xFF;
            $grayscale[$i++] = ($r * 11 + $g * 16 + $b * 5) / 32;
        }
    }

    $result = diffuseDither($grayscale, $w, $h);
//  $result = thresholdDither($grayscale, $w, $h);

    imagedestroy($image);

    return $result;
}

//////////////////////////////////////////////////
// 页模式指令
//////////////////////////////////////////////////

/*
 * 进入页模式。
 */
function enterPageMode()
{
    return "1b4c";
}

/*
 * 设置页模式打印区域。
 * x, y: 打印区域原点（区域的左上角）坐标
 * w, h: 打印区域的宽度和高度
 */
function setPrintAreaInPageMode($x, $y, $w, $h)
{
    $result  = "";
    $result .= "1b57";
    $result .= sprintf("%02x%02x", ($x & 0xFF), (($x >> 8) & 0xFF));
    $result .= sprintf("%02x%02x", ($y & 0xFF), (($y >> 8) & 0xFF));
    $result .= sprintf("%02x%02x", ($w & 0xFF), (($w >> 8) & 0xFF));
    $result .= sprintf("%02x%02x", ($h & 0xFF), (($h >> 8) & 0xFF));
    return $result;
}

/*
 * 设置页模式打印方向。
 * dir: 0:画布不旋转; 1:画布顺时针旋转90度; 2:画布顺时针旋转180度; 3:画布顺时针旋转270度
 */
function setPrintDirectionInPageMode($dir)
{
    if ($dir >= 0 && $dir <= 3)
        return sprintf("1b54%02x", $dir);
    return "";
}

/*
 * 设置页模式下的绝对垂直打印位置。
 */
function setAbsolutePrintPositionInPageMode($n)
{
    if ($n >= 0 && $n <= 65535)
        return sprintf("1d24%02x%02x", ($n & 0xFF), (($n >> 8) & 0xFF));
    return "";
}

/*
 * 设置页模式下的相对垂直打印位置。
 */
function setRelativePrintPositionInPageMode($n)
{
    if ($n >= -32768 && $n <= 32767)
        return sprintf("1d5c%02x%02x", ($n & 0xFF), (($n >> 8) & 0xFF));
    return "";
}

/*
 * 打印并退出页模式。
 */
function printAndExitPageMode()
{
    return "0c";
}

/*
 * 打印（维持在页模式）。
 */
function printInPageMode()
{
    return "1b0c";
}

/*
 * 清除打印区域（维持在页模式）。
 */
function clearInPageMode()
{
    return "18";
}

/*
 * 不打印，退出页模式。
 */
function exitPageMode()
{
    return "1b53";
}

$orderContent = "";

$orderContent .= lineFeed(1);

$orderContent .= setLineSpacing(80);
$orderContent .= setPrintModes(true, true, false);
$orderContent .= setAlignment(ALIGN_CENTER);
$orderContent .= appendText("*** 打印测试 ***\n");

$orderContent .= restoreDefaultLineSpacing();
$orderContent .= setPrintModes(false, false, false);
$orderContent .= setAlignment(ALIGN_LEFT);

/* 设置三列。第一列96点，左对齐；第二列144点，居中对齐；第三列使用剩余的宽度，右对齐 */
setColumnWidths(
    columnWidthWithAlignment(96,  ALIGN_LEFT  ),
    columnWidthWithAlignment(144, ALIGN_CENTER),
    columnWidthWithAlignment(0,   ALIGN_RIGHT ));
setColumnFlags(
    0,
    0,
    COLUMN_FLAG_BW_REVERSE);

$orderContent .= printInColumns("商品名称", "数量\n(单位：随意)", "小计\n(单位：元)");
$orderContent .= lineFeed(1);
$orderContent .= printInColumns("这是\"一个很长的品名\"", "x1000", "￥2020.99");
$orderContent .= lineFeed(1);
$orderContent .= printInColumns("橙子", "【备注：赠品购物满1000,000元送一只】", "￥0.00");
$orderContent .= lineFeed(1);

$orderContent .= setAlignment(ALIGN_CENTER);

/* 打印 CODE128 条码 */
$orderContent .= appendBarcode(HRI_POS_BELOW, 160, 3, 73, "Abc-000789");
$orderContent .= lineFeed(1);

/* 打印二维码 */
$orderContent .= appendQRcode(5, 1, "https://docs.sunmi.com/docking-debugging/商米云打印机合作伙伴对接说明/1-了解一下对接流程/");
$orderContent .= lineFeed(1);

$orderContent .= setAlignment(ALIGN_LEFT);

/* 打印图像文件 */
$orderContent .= appendImage("/home/zhou/dev/php/escpos/cat.jpg");
$orderContent .= lineFeed(1);

/* 页模式打印 */
$orderContent .= setAlignment(ALIGN_CENTER);
$orderContent .= appendText("---- 页模式多区域打印 ----\n");
$orderContent .= setAlignment(ALIGN_LEFT);
$orderContent .= enterPageMode();
// 区域1
$orderContent .= setPrintAreaInPageMode(0, 0, 144, 500);
$orderContent .= setPrintDirectionInPageMode(0);
$orderContent .= appendText("永和九年，岁在癸丑，暮春之初，会于会稽山阴之兰亭，修禊事也。群贤毕至，少长咸集。" .
    "此地有崇山峻岭，茂林修竹；又有清流激湍，映带左右，引以为流觞曲水，列坐其次。\n");
// 区域2
$orderContent .= setPrintAreaInPageMode(156, 0, 144, 500);
$orderContent .= setPrintDirectionInPageMode(2);
$orderContent .= appendText("鎌倉アナウンサーはまず流暢な中国語でアナウンサーとしての豊富な経験を紹介されました。\n");
// 区域3
$orderContent .= setPrintAreaInPageMode(312, 0, 72, 500);
$orderContent .= setPrintDirectionInPageMode(3);
$orderContent .= appendText("Scarlett is a woman who can deal with a nation at war, Atlanta burning.\n");
// 打印并退出页模式
$orderContent .= printAndExitPageMode();

$orderContent .= lineFeed(4);
$orderContent .= cutPaper(false);

pushToPrinter("N403D06A00023", $orderContent);

?>
