# -*- coding: utf-8 -*-

import hashlib, json, requests, time
from urllib import parse, request

APP_ID = '14df7284a4af4c72bb2dc0ca8b820a33'
APP_KEY = 'd0959fea394e437786d8db0ec94977b4'

MAX_COLUMNS = 6

ALIGN_LEFT = 0
ALIGN_CENTER = 1
ALIGN_RIGHT = 2

HRI_POS_ABOVE = 1
HRI_POS_BELOW = 2

COLUMN_FLAG_BW_REVERSE = 1

# 生成签名。
def generateSign(formData):
    keys = sorted(formData)
    s = ''
    for k in keys:
        s += k + '=' + str(formData[k]) + '&'
    s = s[:-1] + APP_KEY
    return hashlib.md5(s.encode('utf-8')).hexdigest().upper()

# 返回一个字符的宽度。0表示未知。
def widthOfChar(c):
    if (c >= 0x00020 and c <= 0x0036f) or \
       (c >= 0x0ff61 and c <= 0x0ff9f):
        return 12
    if (c == 0x02010                 ) or \
       (c >= 0x02013 and c <= 0x02016) or \
       (c >= 0x02018 and c <= 0x02019) or \
       (c >= 0x0201c and c <= 0x0201d) or \
       (c >= 0x02025 and c <= 0x02026) or \
       (c >= 0x02030 and c <= 0x02033) or \
       (c == 0x02035                 ) or \
       (c == 0x0203b                 ):
        return 24
    if (c >= 0x01100 and c <= 0x011ff) or \
       (c >= 0x02460 and c <= 0x024ff) or \
       (c >= 0x025a0 and c <= 0x027bf) or \
       (c >= 0x02e80 and c <= 0x02fdf) or \
       (c >= 0x03000 and c <= 0x0318f) or \
       (c >= 0x031a0 and c <= 0x031ef) or \
       (c >= 0x03200 and c <= 0x09fff) or \
       (c >= 0x0ac00 and c <= 0x0d7ff) or \
       (c >= 0x0f900 and c <= 0x0faff) or \
       (c >= 0x0fe30 and c <= 0x0fe4f) or \
       (c >= 0x1f000 and c <= 0x1f9ff):
        return 24
    if (c >= 0x0ff01 and c <= 0x0ff5e) or \
       (c >= 0x0ffe0 and c <= 0x0ffe5):
        return 24
    return 0

def columnWidthWithAlignment(width, alignment):
    return (width & 0xFFFF) | ((alignment & 3) << 16)

def columnWidth(v):
    return v & 0xFFFF

def columnAlignment(v):
    return (v >> 16) & 3

class EscPosPrinter:
    def __init__(self, dots_per_line=384):
        self._DOTS_PER_LINE = dots_per_line
        self._charHSize = 1
        self._orderData = b''
        self._widthOfColumns = [0] * MAX_COLUMNS
        self._flagOfColumns = [0] * MAX_COLUMNS

    @property
    def DOTS_PER_LINE(self):
        return self._DOTS_PER_LINE

    @property
    def orderData(self):
        return self._orderData

    def clear(self):
        self._orderData = b''

    def pushToPrinter(self, msn):
        timestamp = int(time.time())
        voiceCnt = 1
        voice = '您有新的订单'
        orderCnt = 1
        orderType = 1

        formData = {}
        formData['app_id']    = APP_ID
        formData['msn']       = msn
        formData['timestamp'] = timestamp
        formData['pushId']    = msn + '_{:010d}'.format(timestamp)
        formData['voiceCnt']  = voiceCnt
        formData['voice']     = voice
        formData['orderCnt']  = orderCnt
        formData['orderType'] = orderType
        formData['orderData'] = self._orderData.hex()
        formData['sign']      = generateSign(formData)
        self.clear()

        header = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        req = requests.post('https://openapi.sunmi.com/v1/printer/pushContent', parse.urlencode(formData), headers=header)
        print(json.loads(req.text))

    ##################################################
    # 基本ESC/POS指令
    ##################################################

    # 在小票内容中追加数据。
    def appendRawData(self, data):
        self._orderData += data

    # 在小票内容中追加文本字符串。
    def appendText(self, str):
        self._orderData += str.encode(encoding='utf-8', errors='ignore')

    # [LF] 打印缓冲区里的内容并走纸 n 行。
    def lineFeed(self, n):
        if n <= 0:
            n = 1
        self._orderData += b'\x0a' * n

    # [ESC @] 恢复默认设置 (行间距、字符效果等)。
    def restoreDefaultSettings(self):
        self._charHSize = 1
        self._orderData += b'\x1b\x40'

    # [ESC 2] 恢复默认的行间距。
    def restoreDefaultLineSpacing(self):
        self._orderData += b'\x1b\x32'

    # [ESC 3] 设置行间距。
    def setLineSpacing(self, n):
        if n >= 0 and n <= 255:
            self._orderData += b'\x1b\x33' + n.to_bytes(1, 'little')

    # [ESC !] 选择加粗、倍高、倍宽模式。
    def setPrintModes(self, bold, double_h, double_w):
        n = 0
        if bold:
            n |= 8
        if double_h:
            n |= 16
        if double_w:
            n |= 32
            self._charHSize = 2
        else:
            self._charHSize = 1
        self._orderData += b'\x1b\x21' + n.to_bytes(1, 'little')

    # [GS !] 选择字符高度和宽度倍数。
    def setCharacterSize(self, h, w):
        n = 0
        if h >= 1 and h <= 8:
            n |= (h - 1)
        if w >= 1 and w <= 8:
            n |= (w - 1) << 4
            self._charHSize = w
        self._orderData += b'\x1d\x21' + n.to_bytes(1, 'little')

    # [HT] 跳到下 n 个制表符位置。
    def horizontalTab(self, n):
        if n > 0:
            self._orderData += b'\x09' * n

    # [ESC $] 跳到绝对位置。
    def setAbsolutePrintPosition(self, n):
        if n >= 0 and n <= 65535:
            self._orderData += b'\x1b\x24' + n.to_bytes(2, 'little')

    # [ESC \] 跳到相对位置。
    def setRelativePrintPosition(self, n):
        if n >= -32768 and n <= 32767:
            self._orderData += b'\x1b\x5c' + n.to_bytes(2, 'little')

    # [ESC a] 设置对齐方式。
    def setAlignment(self, n):
        if n >= 0 and n <= 2:
            self._orderData += b'\x1b\x61' + n.to_bytes(1, 'little')

    # [GS B] 设置黑白反转模式。
    def setBlackWhiteReverseMode(self, enabled):
        if enabled:
            self._orderData += b'\x1d\x42\x01'
        else:
            self._orderData += b'\x1d\x42\x00'

    # [ESC {] 设置上下颠倒模式。
    def setUpsideDownMode(self, enabled):
        if enabled:
            self._orderData += b'\x1b\x7b\x01'
        else:
            self._orderData += b'\x1b\x7b\x00'

    # [GS V m] 切纸。
    def cutPaper(self, fullCut):
        if fullCut:
            self._orderData += b'\x1d\x56\x30'
        else:
            self._orderData += b'\x1d\x56\x31'

    # [GS V m n] 延迟切纸。
    # 发送该指令后，打印机不会马上切纸，而是等到后续走纸或打印了 (d + n) 点的距离后再切纸。
    # 其中 d 为切刀与打印头之间的距离，n 为 0~255。
    def postponedCutPaper(self, fullCut, n):
        if n >= 0 and n <= 255:
            if fullCut:
                self._orderData += b'\x1d\x56\x61'
            else:
                self._orderData += b'\x1d\x56\x62'
            self._orderData += n.to_bytes(1, 'little')

    ##################################################
    # 分列打印
    ##################################################

    # 设置每列的宽度和对齐方式。
    def setColumnWidths(self, widths):
        self._widthOfColumns = [0] * MAX_COLUMNS
        remain = self._DOTS_PER_LINE # 未分配的宽度
        for i in range(len(widths)):
            if i == MAX_COLUMNS: # 超过最大支持的列数
                return
            width = columnWidth(widths[i])
            alignment = columnAlignment(widths[i])
            if width == 0 or width > remain: # 使用所有未分配的宽度作为最后一列
                self._widthOfColumns[i] = columnWidthWithAlignment(remain, alignment)
                return
            self._widthOfColumns[i] = widths[i]
            remain -= width

    # 设置每列的标志位。
    def setColumnFlags(self, flags):
        self._flagOfColumns = [0] * MAX_COLUMNS
        for i in range(len(flags)):
            if i == MAX_COLUMNS: # 超过最大支持的列数
                return
            self._flagOfColumns[i] = flags[i]

    # 按当前的列设置进行分列打印。
    def printInColumns(self, texts):
        strcurr = [''] * MAX_COLUMNS
        strrem = [''] * MAX_COLUMNS
        strwidth = [0] * MAX_COLUMNS
        columns = 0

        for i in range(len(texts)):
            if i == MAX_COLUMNS or self._widthOfColumns[i] == 0:
                break
            strcurr[i] = ''
            strrem[i] = texts[i]
            columns += 1

        while True:
            done = True
            pos = 0
            for i in range(columns):
                width = columnWidth(self._widthOfColumns[i])
                if len(strrem[i]) == 0:
                    pos += width
                    continue
                done = False
                strcurr[i] = ''
                strwidth[i] = 0
                j = 0
                while j < len(strrem[i]):
                    c = ord(strrem[i][j])
                    if c == ord('\n'):
                        j += 1 # 丢弃换行符
                        break
                    else:
                        w = widthOfChar(c)
                        if w == 0:
                            c = ord('?')
                            w = 12
                        w *= self._charHSize
                        if strwidth[i] + w > width:
                            break
                        else:
                            strcurr[i] += chr(c)
                            strwidth[i] += w
                    j += 1
                if j < len(strrem[i]):
                    strrem[i] = strrem[i][j:]
                else:
                    strrem[i] = ''

                alignment = columnAlignment(self._widthOfColumns[i])
                if alignment == 1:
                    self.setAbsolutePrintPosition(pos + (width - strwidth[i]) // 2)
                elif alignment == 2:
                    self.setAbsolutePrintPosition(pos + (width - strwidth[i]))
                else:
                    self.setAbsolutePrintPosition(pos)
                if (self._flagOfColumns[i] & COLUMN_FLAG_BW_REVERSE) != 0:
                    self.setBlackWhiteReverseMode(True)
                self.appendText(strcurr[i])
                if (self._flagOfColumns[i] & COLUMN_FLAG_BW_REVERSE) != 0:
                    self.setBlackWhiteReverseMode(False)
                pos += width

            if not done:
                self.lineFeed(1)
            else:
                break

    ##################################################
    # 条码/二维码打印
    ##################################################

    # 在小票内容中追加条码。
    def appendBarcode(self, hri_pos, height, module_size, barcode_type, text):
        text_length = len(text)

        if text_length == 0:
            return
        if text_length > 255:
            text_length = 255
        if height < 1:
            height = 1
        elif height > 255:
            height = 255
        if module_size < 1:
            module_size = 1
        elif module_size > 6:
            module_size = 6

        hri_pos &= 3

        self._orderData += b'\x1d\x48' + hri_pos.to_bytes(1, 'little')
        self._orderData += b'\x1d\x66\x00'
        self._orderData += b'\x1d\x68' + height.to_bytes(1, 'little')
        self._orderData += b'\x1d\x77' + module_size.to_bytes(1, 'little')
        self._orderData += b'\x1d\x6b' + barcode_type.to_bytes(1, 'little') + text_length.to_bytes(1, 'little')
        self._orderData += text.encode(encoding='utf-8', errors='ignore')

    # 在小票内容中追加二维码。
    def appendQRcode(self, module_size, ec_level, text):
        content = text.encode(encoding='utf-8', errors='ignore')
        text_length = len(content)

        if text_length == 0:
            return
        if text_length > 65535:
            text_length = 65535
        if module_size < 1:
            module_size = 1
        elif module_size > 16:
            module_size = 16
        if ec_level < 0:
            ec_level = 0
        elif ec_level > 3:
            ec_level = 3

        ec_level += 48
        text_length += 3

        self._orderData += b'\x1d\x28\x6b\x04\x00\x31\x41\x00\x00'
        self._orderData += b'\x1d\x28\x6b\x03\x00\x31\x43' + module_size.to_bytes(1, 'little')
        self._orderData += b'\x1d\x28\x6b\x03\x00\x31\x45' + ec_level.to_bytes(1, 'little')
        self._orderData += b'\x1d\x28\x6b' + text_length.to_bytes(2, 'little') + b'\x31\x50\x30'
        self._orderData += content
        self._orderData += b'\x1d\x28\x6b\x03\x00\x31\x51\x30'

    ##################################################
    # 页模式指令
    ##################################################

    # 进入页模式。
    def enterPageMode(self):
        self._orderData += b'\x1b\x4c'

    # 设置页模式打印区域。
    # x, y: 打印区域原点（区域的左上角）坐标
    # w, h: 打印区域的宽度和高度
    def setPrintAreaInPageMode(self, x, y, w, h):
        self._orderData += b'\x1b\x57'
        self._orderData += x.to_bytes(2, 'little')
        self._orderData += y.to_bytes(2, 'little')
        self._orderData += w.to_bytes(2, 'little')
        self._orderData += h.to_bytes(2, 'little')

    # 设置页模式打印方向。
    # dir: 0:画布不旋转; 1:画布顺时针旋转90度; 2:画布顺时针旋转180度; 3:画布顺时针旋转270度
    def setPrintDirectionInPageMode(self, dir):
        if dir >= 0 and dir <= 3:
            self._orderData += b'\x1b\x54' + dir.to_bytes(1, 'little')

    # 设置页模式下的绝对垂直打印位置。
    def setAbsolutePrintPositionInPageMode(self, n):
        if n >= 0 and n <= 65535:
            self._orderData += b'\x1d\x24' + n.to_bytes(2, 'little')

    # 设置页模式下的相对垂直打印位置。
    def setRelativePrintPositionInPageMode(self, n):
        if n >= -32768 and n <= 32767:
            self._orderData += b'\x1d\x5c' + n.to_bytes(2, 'little')

    # 打印并退出页模式。
    def printAndExitPageMode(self):
        self._orderData += b'\x0c'

    # 打印（维持在页模式）。
    def printInPageMode(self):
        self._orderData += b'\x1b\x0c'

    # 清除打印区域（维持在页模式）。
    def clearInPageMode(self):
        self._orderData += b'\x18'

    # 不打印，退出页模式。
    def exitPageMode(self):
        self._orderData += b'\x1b\x53'

if __name__ == '__main__':
    printer = EscPosPrinter()

    printer.lineFeed(1)

    printer.setLineSpacing(80)
    printer.setPrintModes(True, True, False)
    printer.setAlignment(ALIGN_CENTER)
    printer.appendText('*** 打印测试 ***\n')

    printer.restoreDefaultLineSpacing()
    printer.setPrintModes(False, False, False)
    printer.setAlignment(ALIGN_LEFT)

    # 设置三列。第一列96点，左对齐；第二列144点，居中对齐；第三列使用剩余的宽度，右对齐
    printer.setColumnWidths((\
        columnWidthWithAlignment(96,  ALIGN_LEFT  ), \
        columnWidthWithAlignment(144, ALIGN_CENTER), \
        columnWidthWithAlignment(0,   ALIGN_RIGHT )))
    printer.setColumnFlags((\
        0, \
        0, \
        COLUMN_FLAG_BW_REVERSE))
    printer.printInColumns(('商品名称', '数量\n(单位：随意)', '小计\n(单位：元)'))
    printer.lineFeed(1)
    printer.printInColumns(('这是\'一个很长的品名\'', 'x1000', '￥2020.99'))
    printer.lineFeed(1)
    printer.printInColumns(('橙子', '【备注：赠品购物满1000,000元送一只】', '￥0.00'))
    printer.lineFeed(1)

    printer.setAlignment(ALIGN_CENTER)

    # 打印 CODE128 条码
    printer.appendBarcode(HRI_POS_BELOW, 160, 3, 73, 'Abc-000789')
    printer.lineFeed(1)

    # 打印二维码
    printer.appendQRcode(5, 1, 'https://docs.sunmi.com/docking-debugging/商米云打印机合作伙伴对接说明/1-了解一下对接流程/')
    printer.lineFeed(1)

    printer.setAlignment(ALIGN_LEFT)

    # 页模式打印
    printer.setAlignment(ALIGN_CENTER)
    printer.appendText('---- 页模式多区域打印 ----\n')
    printer.setAlignment(ALIGN_LEFT)
    printer.enterPageMode()
    # 区域1
    printer.setPrintAreaInPageMode(0, 0, 144, 500)
    printer.setPrintDirectionInPageMode(0)
    printer.appendText('永和九年，岁在癸丑，暮春之初，会于会稽山阴之兰亭，修禊事也。群贤毕至，少长咸集。' +
        '此地有崇山峻岭，茂林修竹；又有清流激湍，映带左右，引以为流觞曲水，列坐其次。\n')
    # 区域2
    printer.setPrintAreaInPageMode(156, 0, 144, 500)
    printer.setPrintDirectionInPageMode(2)
    printer.appendText('鎌倉アナウンサーはまず流暢な中国語でアナウンサーとしての豊富な経験を紹介されました。\n')
    # 区域3
    printer.setPrintAreaInPageMode(312, 0, 72, 500)
    printer.setPrintDirectionInPageMode(3)
    printer.appendText('Scarlett is a woman who can deal with a nation at war, Atlanta burning.\n')
    # 打印并退出页模式
    printer.printAndExitPageMode()

    printer.lineFeed(4)
    printer.cutPaper(False)

    printer.pushToPrinter('N403D06A00023')
